// SPDX-License-Identifier: BSD-3-Clause
// Copyright (c) Contributors to the OpenEXR Project.

// This file is auto-generated by the cmake configure step

#ifndef INCLUDED_IEX_CONFIG_H
#define INCLUDED_IEX_CONFIG_H 1

#pragma once

//
//
// C++ namespace configuration / options

//
// Current internal library namepace name
//
#define IEX_INTERNAL_NAMESPACE_CUSTOM 0
#define IEX_INTERNAL_NAMESPACE Iex_3_1

//
// Current public user namepace name
//

#define IEX_NAMESPACE_CUSTOM 0
#define IEX_NAMESPACE Iex

#endif // INCLUDED_IEX_CONFIG_H
